/*
	Vibrator Control Protocol Daemon
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include <ser.h>
#include <vcp.h>

int main( int argc, char *argv[] )
{
	SIO_INFO *si;
	unsigned char buffer[5];
	int c, i1, i2, i3, i4;

	if( argc<6 ) {
		printf( "To few arguments.\n" );
		return 1;
	}

	if( sscanf( argv[2], "%d", &i1 )!=1 ) {
		printf( "Unable to parse argument 2.\n" );
		return 1;
	}
	if( sscanf( argv[3], "%d", &i2 )!=1 ) {
		printf( "Unable to parse argument 3.\n" );
		return 1;
	}
	if( sscanf( argv[4], "%d", &i3 )!=1 ) {
		printf( "Unable to parse argument 4.\n" );
		return 1;
	}
	if( sscanf( argv[5], "%d", &i4 )!=1 ) {
		printf( "Unable to parse argument 5.\n" );
		return 1;
	}

	si = SIO_Open( argv[1] );
	if( si==NULL ) {
		printf( "Unable to open device.\n" );
		return 1;
	}

	/* Set main port parameters */
	SIO_SetSpeed( si, 9600 );
	SIO_SetDataBits( si, 8 );
	SIO_SetParity( si, SIO_PARITY_NONE );
	SIO_SetStopBits( si, 1 );
	SIO_SetIOMode( si, SIO_IOMODE_DIRECT );
	SIO_WriteSettings( si );

	/* Get unwanted bytes */
	while( SIO_ReadChar( si )!=-1 );

	/* Reset device */
	SIO_RaiseDTR( si ); /* +12V */
	usleep( 1000 );
	SIO_DropDTR( si ); /* -12V */

	SIO_SetReadTimeout( si, 1000000 );

	if( (c=SIO_ReadBuffer( si, buffer, 5 ))!=5 ) {
		printf( "Unable to read 5 bytes. (%d)\n", c );
		return 1;
	}

	if( memcmp( buffer, "BHEF1", 5 )!=0 ) {
		printf( "BHEF1 did not match.\n" );
		return 1;
	}

	if( SIO_WriteChar( si, 0x82 )==-1 ) {
		printf( "Unable to send.\n" );
		return 1;
	}
	if( ((c=SIO_ReadChar( si ))==-1) || (c!=0x82) ) {
		printf( "Did not get correct command ACK.\n" );
		return 1;
	}
	if( SIO_WriteChar( si, i1&0xFF )==-1 ) {
		printf( "Unable to send.\n" );
		return 1;
	}
	if( SIO_WriteChar( si, i2&0xFF )==-1 ) {
		printf( "Unable to send.\n" );
		return 1;
	}
	if( SIO_WriteChar( si, i3&0xFF )==-1 ) {
		printf( "Unable to send.\n" );
		return 1;
	}
	if( SIO_WriteChar( si, i4&0xFF )==-1 ) {
		printf( "Unable to send.\n" );
		return 1;
	}
	if( ((c=SIO_ReadChar( si ))==-1) || (c!=((i1+i2+i3+i4)&0xFF)) ) {
		printf( "Did not get correct data ACK.\n" );
		return 1;
	}

	printf( "Ok.\n" );

	return 0;
}

