/*
	Vibrator Control Firmware
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SER_AVR_H
#define __SER_AVR_H

#define FALSE   0
#define TRUE    !FALSE

//! 1 byte unsiged data type.
typedef unsigned char iu8;
//! 2 byte unsigned data type.
typedef unsigned short iu16;
//! 4 byte unsigned data type.
typedef unsigned long iu32;
//! Boolean data type.
typedef unsigned char bool;

/*! \brief Command: Set

	CMD_SET  ->
             <-  CMD_SET
    Level    ->
             <-  Level
*/
#define CMD_SET		0x81

/* \brief Command: PWM CONF

	CMD_PWMCONF   ->
                  <- CMD_PWMCONF
	IN_LOW        ->
	IN_HIGH       ->
	OUT_LOW       ->
	OUT_HIGH      ->
                  <- (IN_LOW+IN_HIGH+OUT_LOW+OUT_HIGH) & 0xFF
*/
#define CMD_PWMCONF	0x82

#define CMD_LED_ON	0x91
#define CMD_LED_OFF	0x92
#define CMD_ABS		0x93

#define RSP_ACK		0x06
#define RSP_NAK		0x15

/* LED */
#define	LED_USE
#define LED_DDR		DDRD
#define LED_PORT	PORTD
#define LED_PIN		PD7

/* PWM Port */
#define PWM_DDR		DDRB
#define PWM_PIN		PB1

/* UBR */
/* May need adjusting */
//#define UBR_9600	23	/* 3.686MHz */
#define UBR_9600	39	/* 6 MHz */

/* EEPROM Addresses */
#define ADDR_INITED		1
#define ADDR_PWMCONF	2

void hal_init( void );
bool hal_eeprom_read( iu8 *dst, iu16 src, iu8 len );
bool hal_eeprom_write( iu16 dst, iu8 *src, iu8 len );
void hal_sendByte( iu8 b );
iu8 hal_recByte( void );
void hal_setPwm( iu8 width );
void hal_led( bool on );
void hal_die( void );

#endif

