/*
	Vibrator Control Protocol Daemon
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ctype.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/time.h>
#include <unistd.h>

#include <proc.h>
#include <ser.h>
#include <vcp.h>

#if 0
void vcp_dump_state( S_VCP_STATE *vs )
{
	fprintf( stderr, "vs.type: %d\n", vs->type );
	fprintf( stderr, "vs.cs: %d\n", vs->cs );
	fprintf( stderr, "vs.si: %.8X\n", vs->si );
	fprintf( stderr, "strlen(vs.buffer): %d\n", strlen(vs->buffer) );
	fprintf( stderr, "vs.hostname: [%s]\n", vs->hostname );
	fprintf( stderr, "vs.username: [%s]\n", vs->username );
	fprintf( stderr, "vs.auth.login: [%s]\n", vs->auth.login );
	fprintf( stderr, "vs.auth.password: [%s]\n", vs->auth.password );
}
#endif

int vcp_init( S_VCP_STATE *vs, char *port )
{
	switch( vs->type ) {
	case VIB_TYPE_SER:
		return ser_init( vs, port );
	case VIB_TYPE_DUMMY:
		vs->func.vibr=vcp_vibr_dummy;
		vs->func.intv=vcp_intv_dummy;
		return TRUE;
	default:
	}

	return FALSE;
}

int vcp_control( S_VCP_STATE *vs )
{
	int i, len, err;
	char *c, *c2;
	S_VCP_INTV	intv, list, *currlist;

	snprintf( vs->buffer, VCP_BUFLEN, VCP_RETCODE_READY, VCP_VERSION );
	write( vs->cs, vs->buffer, strlen(vs->buffer) );

	for(;;) {
		len = read( vs->cs, vs->buffer, VCP_BUFLEN );
		vs->buffer[VCP_BUFLEN]=0;

		if( len==0 ) continue;

		if( len<5 ) goto unknown;

		for( i=0; i<4; i++ ) vs->buffer[i]=toupper(vs->buffer[i]);

		if( strncmp( "AUTH", vs->buffer, 4 )==0 ) {
			vs->auth.verified=0;

			/* Separate arguments */
			c = &vs->buffer[5];
			while( *c==' ' ) c++;
			/* c should now point to the login */
			if( (*c==0) || (*c=='\n') || (*c=='\r') ) {
				syslog( LOG_WARNING, "AUTH unsuccessful because of parameter error." );
				write( vs->cs, VCP_RETCODE_SYNTAX, strlen(VCP_RETCODE_SYNTAX) );
				continue;
			}

			c2 = strchr( c, ' ' );
			if( c2!=NULL ) {
				*c2=0;
				c2++;
				while( *c2==' ' ) c2++;
				if( strchr( c2, '\n' )!=NULL ) *(strchr( c2, '\n' ))=0;
				if( strchr( c2, '\r' )!=NULL ) *(strchr( c2, '\r' ))=0;
				if( strchr( c2, ' ' )!=NULL ) *(strchr( c2, ' ' ))=0;
			}
			/* c2 should now point to the password */
			if( (c2==NULL) || (*c2==0) || (*c2=='\n')  || (*c2=='\r') ) {
				syslog( LOG_WARNING, "AUTH unsuccessful because of parameter error." );
				write( vs->cs, VCP_RETCODE_SYNTAX, strlen(VCP_RETCODE_SYNTAX) );
				continue;
			}

			/* Compare login */
			if( strncmp( c, &vs->auth.login[0], sizeof(vs->auth.login) )!=0 ) {
				syslog( LOG_WARNING, "AUTH wrong login: %s.", c );
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}
			/* Compare password */
			if( strncmp( c2, &vs->auth.password[0], sizeof(vs->auth.password) )!=0 ) {
				syslog( LOG_WARNING, "AUTH wrong password (login %s).", c );
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}

			vs->auth.verified=1;

			syslog( LOG_INFO, "AUTH %s successful.", c );
			write( vs->cs, VCP_RETCODE_OK, strlen(VCP_RETCODE_OK) );

			continue;
		}

		if( strncmp( "HELP", vs->buffer, 4 )==0 ) {
			write( vs->cs, VCP_HELP, strlen(VCP_HELP) );

			write( vs->cs, VCP_RETCODE_OK, strlen(VCP_RETCODE_OK) );

			continue;
		}

		if( strncmp( "INFO", vs->buffer, 4 )==0 ) {
			if( !vs->auth.verified ) {
				write( vs->cs, VCP_RETCODE_UNAUTH, strlen(VCP_RETCODE_UNAUTH) );
				continue;
			}

			snprintf( &vs->buffer[0], VCP_BUFLEN, "USER %s\n", vs->username );
			write( vs->cs, vs->buffer, strlen(&vs->buffer[0]) );

			if( vs->mode==VIB_MODE_VIBR ) {
				snprintf( &vs->buffer[0], VCP_BUFLEN, "VIBR %d\n", vs->vibr );
				write( vs->cs, vs->buffer, strlen(&vs->buffer[0]) );
			} else if( vs->mode==VIB_MODE_INTV ) {
				write( vs->cs, "INTV", strlen("INTV") );
				currlist=vs->intv;
				while( currlist!=NULL ) {
					snprintf( &vs->buffer[0], VCP_BUFLEN, " %d,%d",
						currlist->time, currlist->level );
					write( vs->cs, vs->buffer, strlen(&vs->buffer[0]) );
					currlist=currlist->next;
				}
				write( vs->cs, "\n", strlen("\n") );
			}

			write( vs->cs, VCP_RETCODE_OK, strlen(VCP_RETCODE_OK) );

			continue;
		}

		if( strncmp( "INTV", vs->buffer, 4 )==0 ) {
			//write( vs->cs, VCP_RETCODE_CNIMPL, strlen(VCP_RETCODE_CNIMPL) );

			if( !vs->auth.verified ) {
				write( vs->cs, VCP_RETCODE_UNAUTH, strlen(VCP_RETCODE_UNAUTH) );
				continue;
			}

			/* Parse arguments */
			c = vs->buffer+4;
			list.next=NULL;
			currlist=&list;
			err=FALSE;
			while( (i=sscanf( c, "%d,%d", &intv.time, &intv.level))==2 ) {
				/* Get new struct */
				currlist->next=malloc(sizeof(S_VCP_INTV));
				currlist=currlist->next;

				if( (intv.time<0) || (intv.level<0) || (intv.level>255) ) {
					err=TRUE;
					break;
				}

				currlist->time=intv.time;
				currlist->level=intv.level;
				currlist->next=NULL;

				/* Advance pointer */
				while( (isdigit(*c) || isspace(*c)) && (*c!=0) ) c++;
				if( *c==',' ) c++;
				while( isdigit(*c) && (*c!=0) ) c++;
				/* parameter check */
				if( !(isspace(*c) || (*c==0)) ) {
					err=TRUE;
					break;
				}
			}

			if( err ) {
				write( vs->cs, VCP_RETCODE_SYNTAX, strlen(VCP_RETCODE_SYNTAX) );
				continue;
			}

			/* Delete old */
			while( vs->intv!=NULL ) {
				S_VCP_INTV **last=&vs->intv;
				S_VCP_INTV *curr=*last;

				while( curr->next!=NULL ) {
					last=&curr->next;
					curr=curr->next;
				}

				free( curr );
				*last=NULL;
			}
			/* Attach new */
			vs->intv=list.next;
			vs->currintv=NULL;

			if( (vs->mode==VIB_MODE_INTV) && !proc_kill( vs ) ) {
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}

			vs->mode=VIB_MODE_INTV;

			if( !proc_fork( vs ) ) {
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}

			write( vs->cs, VCP_RETCODE_OK, strlen(VCP_RETCODE_OK) );

			continue;
		}

		if( strncmp( "QUIT", vs->buffer, 4 )==0 ) {
			if( vs->mode==VIB_MODE_INTV ) proc_kill( vs );
			vs->func.vibr( vs, 0 );

			write( vs->cs, VCP_RETCODE_CLOSE, strlen(VCP_RETCODE_CLOSE) );

			return TRUE;
		}

		if( strncmp( "VIBR", vs->buffer, 4 )==0 ) {
			if( !vs->auth.verified ) {
				write( vs->cs, VCP_RETCODE_UNAUTH, strlen(VCP_RETCODE_UNAUTH) );
				continue;
			}

			if( (sscanf( vs->buffer, "VIBR %d", &i )!=1) || (i<0) || (i>255) ) {
				write( vs->cs, VCP_RETCODE_SYNTAX, strlen(VCP_RETCODE_SYNTAX) );
				continue;
			}

			if( (vs->mode==VIB_MODE_INTV) && !proc_kill( vs ) ) {
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}

			vs->mode=VIB_MODE_VIBR;

			if( !vs->func.vibr( vs, i ) ) {
				write( vs->cs, VCP_RETCODE_ERROR, strlen(VCP_RETCODE_ERROR) );
				continue;
			}

			write( vs->cs, VCP_RETCODE_OK, strlen(VCP_RETCODE_OK) );

			continue;
		}

unknown:
		write( vs->cs, VCP_RETCODE_UNKN, strlen(VCP_RETCODE_UNKN) );
	}

	return TRUE;
}

int vcp_vibr_dummy( S_VCP_STATE *vs, int level )
{
	syslog( LOG_WARNING, "%s: Setting to level %d", __FUNCTION__, level );

	vs->vibr=level;

	return TRUE;
}

/* define an alarm signal handler. */
void vcp_catch_alarm(int sig_num)
{
}

void vcp_intv_dummy( S_VCP_STATE *vs )
{
	int time, level;
	struct itimerval itv;

	syslog( LOG_WARNING, "%s: Doing INTV function", __FUNCTION__ );

	if( vs->intv==NULL ) exit( 1 );

	vs->currintv=vs->intv;

	/* Setup signal handler */
	signal( SIGALRM, vcp_catch_alarm );
	itv.it_interval.tv_sec=0;
	itv.it_interval.tv_usec=0;

	for(;;) {
		if( vs->currintv==NULL ) vs->currintv=vs->intv;

		time=vs->currintv->time;
		level=vs->currintv->level;

		vs->currintv=vs->currintv->next;

		/* Set level */
		if( !vcp_vibr_dummy( vs, level ) ) exit( 2 );

		/* Wait */
		itv.it_value.tv_sec=time/1000;
		itv.it_value.tv_usec=(time%1000)*1000;
		setitimer( ITIMER_REAL, &itv, NULL );
		pause();
	}
}

